import isEqual from 'lodash.isequal';
import { diffJson } from 'diff';
import colors from 'colors';
import { Phases, symbol } from './constants';

var stringifyError = function stringifyError(error) {
  return error == null ? 'no error' : error.toString();
};

var stringifyDiff = function stringifyDiff(expected, result) {
  return diffJson(expected, result, {
    undefinedReplacement: '<undefined>'
  }).map(function (change) {
    var color = colors.gray;
    var prefix = '';

    if (change.added) {
      color = colors.green;
      prefix = '+';
    } else if (change.removed) {
      color = colors.red;
      prefix = '-';
    }

    return color("" + prefix + change.value);
  }).join('');
};

var checkState = function checkState(state) {
  if (state.phase < Phases.COMMAND) {
    return new TypeError('invalid phase');
  }

  if (!state.isDefaultAssertion) {
    return new TypeError('assertion already assigned');
  }

  return null;
};

export var shouldProduceEvent = function shouldProduceEvent(context, expectedEvent) {
  var state = context[symbol];
  var invalidStateError = checkState(state);

  if (invalidStateError) {
    throw invalidStateError;
  }

  state.assertion = function (resolve, reject, result, error) {
    if (error) {
      return reject(new Error("expected an event, but received an error " + error));
    }

    if (!isEqual(result, expectedEvent)) {
      return reject(new Error("shouldProduceEvent assertion failed:\n " + stringifyDiff(expectedEvent, result)));
    }

    return resolve(result);
  };

  state.isDefaultAssertion = false;
  return context;
};
export var shouldThrow = function shouldThrow(context, expectedError) {
  var state = context[symbol];
  var invalidStateError = checkState(state);

  if (invalidStateError) {
    throw invalidStateError;
  }

  state.assertion = function (resolve, reject, result, error) {
    if (!error) {
      return reject(new Error("expected an error, but received no error received"));
    }

    if (!isEqual(error, expectedError)) {
      return reject(new Error("expected error " + stringifyError(expectedError) + ", but received " + stringifyError(error)));
    }

    return resolve(result);
  };

  state.isDefaultAssertion = false;
  return context;
};
//# sourceMappingURL=aggregate-assertions.js.map