"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.shouldThrow = exports.shouldProduceEvent = void 0;

var _lodash = _interopRequireDefault(require("lodash.isequal"));

var _diff = require("diff");

var _colors = _interopRequireDefault(require("colors"));

var _constants = require("./constants");

var stringifyError = function stringifyError(error) {
  return error == null ? 'no error' : error.toString();
};

var stringifyDiff = function stringifyDiff(expected, result) {
  return (0, _diff.diffJson)(expected, result, {
    undefinedReplacement: '<undefined>'
  }).map(function (change) {
    var color = _colors["default"].gray;
    var prefix = '';

    if (change.added) {
      color = _colors["default"].green;
      prefix = '+';
    } else if (change.removed) {
      color = _colors["default"].red;
      prefix = '-';
    }

    return color("" + prefix + change.value);
  }).join('');
};

var checkState = function checkState(state) {
  if (state.phase < _constants.Phases.COMMAND) {
    return new TypeError('invalid phase');
  }

  if (!state.isDefaultAssertion) {
    return new TypeError('assertion already assigned');
  }

  return null;
};

var shouldProduceEvent = function shouldProduceEvent(context, expectedEvent) {
  var state = context[_constants.symbol];
  var invalidStateError = checkState(state);

  if (invalidStateError) {
    throw invalidStateError;
  }

  state.assertion = function (resolve, reject, result, error) {
    if (error) {
      return reject(new Error("expected an event, but received an error " + error));
    }

    if (!(0, _lodash["default"])(result, expectedEvent)) {
      return reject(new Error("shouldProduceEvent assertion failed:\n " + stringifyDiff(expectedEvent, result)));
    }

    return resolve(result);
  };

  state.isDefaultAssertion = false;
  return context;
};

exports.shouldProduceEvent = shouldProduceEvent;

var shouldThrow = function shouldThrow(context, expectedError) {
  var state = context[_constants.symbol];
  var invalidStateError = checkState(state);

  if (invalidStateError) {
    throw invalidStateError;
  }

  state.assertion = function (resolve, reject, result, error) {
    if (!error) {
      return reject(new Error("expected an error, but received no error received"));
    }

    if (!(0, _lodash["default"])(error, expectedError)) {
      return reject(new Error("expected error " + stringifyError(expectedError) + ", but received " + stringifyError(error)));
    }

    return resolve(result);
  };

  state.isDefaultAssertion = false;
  return context;
};

exports.shouldThrow = shouldThrow;
//# sourceMappingURL=aggregate-assertions.js.map